#!/usr/bin/env python
#-*-coding:utf-8-*-
from infopython.util import dttools

# "THE WISKEY-WARE LICENSE":
# <jbc.develop@gmail.com> wrote this file. As long as you retain this notice you
# can do whatever you want with this stuff. If we meet some day, and you think
# this stuff is worth it, you can buy me a WISKEY in return Juan BC


#===============================================================================
# DOCS
#===============================================================================

"""


"""


#===============================================================================
# IMPORTS
#===============================================================================

import datetime
import inspect

import interpolator
import media
import theory
from util import frozendict


#===============================================================================
# AGENDA
#===============================================================================

class AgendaSetting(theory.Theory):


    def __init__(self, name, mtype, medias=[],
                 audience_valuator=None, impact_valuator=None,
                 audience_interpolator=None, impact_interpolator=None):

        assert audience_valuator == None \
                or hasattr(audience_valuator, "__call__")

        assert impact_valuator == None \
                or hasattr(impact_valuator, "__call__")

        assert isinstance(audience_interpolator,
                          interpolator.AbstractInterpolator) \
               or audience_interpolator == None

        assert isinstance(impact_interpolator,
                           interpolator.AbstractInterpolator) \
               or impact_interpolator == None

        super(AgendaSetting, self).__init__(name, mtype, medias)
        self._audience_valuator = audience_valuator
        self._impact_valuator = impact_valuator
        self._audience_interpolator = audience_interpolator
        self._impact_interpolator = impact_interpolator

    def _audience_of(self, m):
        aud = None
        if self.audience_valuator != None:
            aud = self.audience_valuator(m)
        if  aud == None and self.audience_interpolator != None:
            x = None
            if self.impact_valuator:
                x = self.impact_valuator(m)
            aud = self.audience_interpolator(x or 0.0)
        return aud or 0.0

    def _impact_of(self, m):
        imp = None
        if self.impact_valuator != None:
            imp = self.impact_valuator(m)
        if imp == None and self.impact_interpolator != None:
            x = None
            if self.audience_valuator:
                x = self.audience_valuator(m)
            imp = self.impact_interpolator(x or 0.0)
        return imp or 0.0

    def _value_of(self, m):
        audience = self._audience_of(m)
        impact = self._impact_of(m)
        return audience * impact

    def _data_of(self, m, rank=None):
        return frozendict.FrozenDict({
            "impact": self._impact_of(m),
            "audience": self._audience_of(m),
            "value": self._value_of(m),
            "media": m,
            "datetime": datetime.datetime.now(),
            "agenda": self,
            "rank": rank,
        })

    #===========================================================================
    # REIMPLEMENTED!
    #===========================================================================

    def __iter__(self):
        for rank, m in enumerate(super(AgendaSetting, self).__iter__()):
            yield self._data_of(m, rank)
            
    def __getitem__(self, i):
        return self._data_of(self._medias[i], i)

    def sort(self, policy="value", reverse=False):
        assert policy in ("value", "audience", "impact")
        datas = list(self)
        datas.sort(key=lambda d: d[policy], reverse=reverse)
        self.clear()
        for d in datas:
            self.add(d["media"])
        self._sorted_at = datetime.datetime.now()
        
    def dict_resume(self):
        resume = super(AgendaSetting, self).dict_resume() 
        
        # delete the old media list
        resume["medias"] = []
        for d in self:
            m_resume = dict(
                (k, str(v)) for k, v in d.items()
                            if k not in ("agenda", "media", "datetime")
            )
            m_resume["datetime"] = dttools.strftime(d["datetime"])
            m_resume["name"] = d["media"].name
            resume["medias"].append(m_resume)
        return resume
        
    #===========================================================================
    # properties
    #===========================================================================

    @property
    def audience_interpolator(self):
        return self._audience_interpolator

    @property
    def impact_interpolator(self):
        return self._impact_interpolator

    @property
    def audience_valuator(self):
        return self._audience_valuator

    @property
    def impact_valuator(self):
        return self._impact_valuator


#===============================================================================
# FUNCTIONS
#===============================================================================

def sorted_medias(agendas, policy="value", reverse=False):
    """This method is used for sort medias of diferent types in diferent agendas

    params:
        ``agendas``
            An iterable of agendas.

        ``policy``
            The policy to be sorted [value/audience/impact].

        ``reverse``
            First the highest value

    """
    assert policy in ("value", "audience", "impact")

    mlist = []
    for a in agendas:
        for v in a:
            mlist.append(v)

    key = lambda m: m[policy]
    
    ranked = []
    for rank, m in enumerate(sorted(mlist, key=key, reverse=reverse)):
        mdict = dict(m)
        mdict["rank"] = rank
        ranked.append(frozendict.FrozenDict(mdict))

    return ranked


#===============================================================================
# MAIN
#===============================================================================

if __name__ == "__main__":
    print __doc__

