#!/usr/bin/env python
#-*-coding:utf-8-*-

# "THE WISKEY-WARE LICENSE" (Revision 42):
# <jbc.develop@gmail.com> wrote this file. As long as you retain this notice you
# can do whatever you want with this stuff. If we meet some day, and you think
# this stuff is worth it, you can buy me a wiskey in return Juan BC


#===============================================================================
# DOCS
#===============================================================================

"""This module contains all utilities for interpolate values.

Many of the classes are wrappers for simplify the interpolator of sycpy

"""


#===============================================================================
# IMPORTS
#===============================================================================

import abc

from scipy import interpolate as scipy_interpolate


#===============================================================================
# INTERPOLATORS
#===============================================================================

_registered_inter = {}

def register_interpolator(interpolator_name, interpolator_class):
    """Register an iterpolator

    Can be used as decorator:

    ::
        @register_interpolator(interpolator_name)
        class MyInterpolator(AbstractInterpolator):
            ...

    params:
        ``interpolator_name``
            A name for identify an interpolator
        ``interpolator_class``
            A interpolator type

    returns ``interpolator class``

    """
    assert isinstance(interpolator_name, basestring)
    assert issubclass(interpolator_class, AbstractInterpolator)
    _registered_inter[interpolator_name] = interpolator_class



def interpolator(interpolator_name, xys):
    """Returns an interpolator configured with the xs and ys values

    params:

        ``interpolator_name``
            A name of a registered an interpolator.
        ``xys``
            A set of tuples with the formar *(x, y)* values.

    @return: the interpolator instance

    """
    return _registered_inter[interpolator_name](xys)


def interpolator_names():
    """Return all registered interpolator names"""
    return _registered_inter.keys()


#===============================================================================
# BASE CLASS
#===============================================================================

class AbstractInterpolator(object):
    """This is the base class of all interpolators.

    """


    __metaclass__ = abc.ABCMeta

    def __init__(self, xys=None):
        """Creates a new instance of an interpolator

        params:

            ``xys``
                a iterable of tuples like
                ``[(x0, y0), (x1, y1), ... ,(xn, yn)]``

        """
        if xys != None:
            self.add_points(xys)
            
    def __call__(self, x):
        """i.__call__(x) <==> i(x) <==> i.interpolate(x)
        
        """
        return self.interpolate(x)

    @abc.abstractmethod
    def add_points(self, xys):
        """Adds more points to the interpolator

        params:

            ``xys``
                a iterable of tuples like
                ``[(x0, y0), (x1, y1), ... ,(xn, yn)]``

        """
        raise NotImplementedError()

    @abc.abstractmethod
    def interpolate(self, x):
        """Evaluate a interpolation of the given value

        params:

            ``x``
                scalar

        return:

            Value obtained to interpolate x

        """
        raise NotImplementedError()


#===============================================================================
# PIECE WISE POLYNOMIAL
#===============================================================================

class PieceWisePolynomial(AbstractInterpolator):

    # copy docs from scipy
    __doc__ = scipy_interpolate.PiecewisePolynomial.__doc__

    def add_points(self, xys):
        coord = list(xys)
        coord.sort()
        xs = []
        ys = []
        for x, y in coord:
            xs.append(x)
            ys.append([y])
        if hasattr(self, "_interpolator"):
            self._interpolator.extend(coord)
        else:
            self._interpolator = scipy_interpolate.PiecewisePolynomial(xs, ys)

    def interpolate(self, x):
        return self._interpolator.derivative(x, 0)

register_interpolator("piecewise_polynomial", PieceWisePolynomial)


#===============================================================================
# LINEAR SQUARE
#===============================================================================

class LinearLeastSquares(AbstractInterpolator):
    """In statistics, simple linear regression is the least squares estimator of
    a linear regression model with a single explanatory variable. In other
    words, simple linear regression fits a straight line through the set of n
    points in such a way that makes the sum of squared residuals of the model
    (that is, vertical distances between the points of the data set and the
    fitted line) as small as possible.

    """

    def _set_up(self, xys):

        self._xys = list(xys)
        
        n = len(xys)
        sum_xy = 0.0
        sum_x = 0.0
        sum_y = 0.0
        sum_x_sq = 0.0
        for xi, yi in self._xys:
            sum_xy += xi * yi
            sum_x += xi
            sum_y += yi
            sum_x_sq += xi ** 2 
            
        self._m = (n * sum_xy - sum_x * sum_y) / (n * sum_x_sq - sum_x ** 2)
        self._b = (sum_y - self._m * sum_x) / n
        
    def add_points(self, xys):
        if hasattr(self, "_xys"):
            self._setup(self._xys + list(xys))
        else:
            self._set_up(xys)

    def interpolate(self, x):
        return self._b + self._m * x;

register_interpolator("linear_least_squares", LinearLeastSquares)


#===============================================================================
# EXPERT
#===============================================================================

class Expert(AbstractInterpolator):
    """Fake interpolator in the constructor receives a list of points with their
    respective values ​​to return.
    
    """

    def add_points(self, xys):
        if not hasattr(self, "_points"):
            self._points = {}
        for x, y in xys:
            if x not in self._points:
                self._points[x] = y

    def interpolate(self, x):
        return self._points.get(x, None)

    def get(self, x, d=None):
        """Return the "interpolate" value of ``x`` or ``d``
        ``d`` defaults to None.
        
        """
        return self._points.get(x, d)


register_interpolator("expert", Expert)


#===============================================================================
# MAIN
#===============================================================================

if __name__ == "__main__":
    print __doc__
