#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# THE WISKEY-WARE LICENSE
# -----------------------
#
# "THE WISKEY-WARE LICENSE":
# <jbc dot develop at gmail dot com> wrote this file. As long as you retain this notice you
# can do whatever you want with this stuff. If we meet some day, and you think
# this stuff is worth it, you can buy me a wiskey in return juan
#

#===============================================================================
# DOC
#===============================================================================

"""Module for serve theories by http

"""

#===============================================================================
# IMPORT
#===============================================================================

import json

import theory
import _metadata

from util import xmlparser
from libs import bottle


#===============================================================================
# FORMATS
#===============================================================================

FORMATS = {
    "json": {
        "content-type": "application/json;charset=utf-8",
        "dumps": json.dumps
    },
    "xml": {
        "content-type": "text/xml",
        "dumps": xmlparser.dumps
    }
}

INDEX_HTML = """
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
    <head>
        <title>Infopython Server v.{version}</title>
        <meta http-equiv="content-type" 
            content="text/html;charset=utf-8"/>
    </head>
    <body>
        <h1>Infopython Server {msg}</h1>
        <a href="{url}">HomePage</a>
        <a href="mailto:{email}">Contact</a>
        <p>Powered by <a href="http://bottlepy.org/">bottle</a></p>
    </body>
</html>
"""

#===============================================================================
# Callback CLASS
#===============================================================================

class CallBack(object):
    
    def __init__(self, responser):
        self._r = responser
        
    def __call__(self, format, *args, **kwargs):
        try:
            parsers = FORMATS[format]
        except KeyError as e:
            raise bottle.HTTPError(400, 'Invalid format: ' + e.args[0])
        else:
            try:
                bottle.response.content_type = parsers["content-type"]
                data = self._r(*args, **kwargs)
                return parsers["dumps"](data)
            except Exception as e:
                data = {"error": str(e)}
                return parsers["dumps"](data)


#===============================================================================
# SERVER CLASS
#===============================================================================

class Server(bottle.Bottle):
    """

    """
    
    def __init__(self, name="infopython", theories=(), *args, **kwargs):
        super(Server, self).__init__(*args, **kwargs)
        self._name = name
        self._theories = {}
        for t in theories:
            assert isinstance(t, theory.Theory)
            self._theories[t.name] = t
        
        # 0.0.0.0:8080/server/test -> html
        self.route(
            "/{}".format(self._name),
            callback=self.index_html
        )
        # 0.0.0.0:8080/server/json/test -> json
        self.route(
            "/{}.<format>".format(self._name),
            callback=CallBack(self.index)
        )
        # 0.0.0.0:8080/server/json/theories -> json
        self.route(
            "/{}/list_theories.<format>".format(self._name),
            callback=CallBack(self.list_theories)
        )
        # 0.0.0.0:8080/server/json/theories -> json
        self.route(
            "/{}/show_theory/<theory_name>.<format>".format(self._name),
            callback=CallBack(self.show_theory)
        )

    #=================================================
    # ROUTES
    #=================================================
    
    def index(self):
        return {self._name: "WORKS!"}
        
    def index_html(self):
        vrs = {
            "msg": "'{}' - {}".format(*self.index().items()[0]),
            "version": ".".join(_metadata.VERSION),
            "url": _metadata.URL,
            "email": _metadata.EMAIL,
        }
        html = INDEX_HTML.format(**vrs)
        return html
        
    def list_theories(self):
        tlist = []
        for k, v in self._theories.items():
            data = {
                "name": k,
                "type": type(v).__name__,
                "media_type": v.media_type.__name__ 
            }
            tlist.append(data)
        return tlist
    
    def show_theory(self, theory_name):
        if theory_name in self._theories:
            return self._theories[theory_name].dict_resume()
        msg = "theory {} not found".format(theory_name)
        raise Exception(msg)
    
    #=================================================
    # PROPERTIES
    #=================================================
    
    @property
    def name(self):
        return self._name
        
    @property
    def theories(self):
        return self._theories.values()
    

#===============================================================================
# MAIN
#===============================================================================

if __name__ == "__main__":
    print(__doc__)
