#!/usr/bin/env python
# -*- coding: utf-8 -*-

# "THE WISKEY-WARE LICENSE":
# <jbc.develop@gmail.com> wrote this file. As long as you retain this notice you
# can do whatever you want with this stuff. If we meet some day, and you think
# this stuff is worth it, you can buy me a WISKEY in return Juan BC

#===============================================================================
# DOC
#===============================================================================

"""Test for infopython.agenda_setting

"""

#===============================================================================
# IMPORTS
#===============================================================================

import unittest
import random
import hashlib

from infopython.util import dttools
from infopython import media, agenda_setting, interpolator


#===============================================================================
# TEST
#===============================================================================

class AgendaSettingTests(unittest.TestCase):
    
    def setUp(self):
        self.mtype, self.medias, self.ag = get_data()

    def test_iter(self):
        for rank, m in enumerate(self.ag):
            self.assertIn(m["media"], self.medias)
            mm = {
                "impact": self.ag._impact_of(m["media"]),
                "audience": self.ag._audience_of(m["media"]),
                "value": self.ag._value_of(m["media"]),
                "media": m["media"],
                "datetime": m["datetime"],
                "agenda": self.ag,
                "rank": rank,
            }
            self.assertEqual(dict(m), mm)            
            
    def test_dict_resume(self):
        medias = []
        for m in list(self.ag):
            m_resume = dict(
                (k, str(v)) for k, v in m.items()
                            if k not in ("agenda", "media", "datetime")
            )
            m_resume["datetime"] = ""
            m_resume["name"] = m["media"].name
            medias.append(m_resume)
        
        resume = self.ag.dict_resume()
        for m in resume["medias"]:
            m["datetime"] = ""
            
        mm = {
            "media_type": self.mtype.__name__,
            "medias": medias,
            "type": agenda_setting.AgendaSetting.__name__,
            "name": "ag",
            "datetime": resume["datetime"],
            "sorted_at": resume["sorted_at"]
        }
        
        self.assertEqual(mm, resume)
        
    def test_sort(self):    
        
        for policy in ("value", "audience", "impact"):
            for reverse in (True, False):
                
                self.ag.sort(policy=policy, reverse=reverse)

                last_rank = 0
                last_policy = self.ag[0][policy]
                for m in self.ag:
                    if reverse:
                        self.assertLessEqual(m[policy], last_policy)
                    else:
                        self.assertGreaterEqual(m[policy], last_policy)
                    self.assertGreaterEqual(m["rank"], last_rank)
                    last_policy = m[policy]
                    last_rank = m["rank"]

                    
#===============================================================================
# FUNCTIONS TEST
#===============================================================================

class AgendaSettingFunctionTests(unittest.TestCase):
    
    def setUp(self):
        self.mtype0, self.medias0, self.ag0 = get_data()
        self.mtype1, self.medias1, self.ag1 = get_data()
        
    def test_sorted_medias(self):
        
        for policy in ("value", "audience", "impact"):
            for reverse in (True, False):
                
                sorted_medias = agenda_setting.sorted_medias(
                    [self.ag0, self.ag1],
                    policy=policy, reverse=reverse
                )
                                                
                last_rank = 0
                last_policy = sorted_medias[0][policy]
                for m in sorted_medias:
                    if reverse:
                        self.assertLessEqual(m[policy], last_policy)
                    else:
                        self.assertGreaterEqual(m[policy], last_policy)
                    self.assertGreaterEqual(m["rank"], last_rank)
                    last_policy = m[policy]
                    last_rank = m["rank"]
 
                    

        
        
        
        
        
#===============================================================================
# SUPPORT
#===============================================================================
    
def get_data():
    """Creates and return a random mtype, medias and agenda
    
    """
    name = hashlib.sha1(
        str(random.random())
    ).hexdigest()
    mtype = type(name, (media.Media,), {})
   
    buff = set()
    for _ in range(random.randint(2, 100)):
        name = hashlib.sha1(
            str(random.random())
        ).hexdigest()
        buff.add(mtype(name))
    medias = list(buff)

    exclude_av = set([
            random.choice(medias)
            for _ in range(random.randint(0, len(medias) / 2))
    ])
    exclude_iv = set([
            random.choice(medias)
            for _ in range(random.randint(0, len(medias) / 2))
    ])
    audience_i = interpolator.Expert([
            (medias.index(m), sum(ord(c) for c in m.name) * random.random())
            for m in medias
    ])
    impact_i = interpolator.Expert([
            (sum(ord(c) for c in m.name), medias.index(m) * random.random())
            for m in medias
    ])
    ag = agenda_setting.AgendaSetting(
            name="ag", mtype=mtype, medias=medias,
            audience_valuator=
                lambda m: None if m in exclude_av else sum(ord(c) for c in m.name),
            impact_valuator=
                lambda m: None if m in exclude_iv else medias.index(m),
            audience_interpolator=audience_i,
            impact_interpolator=impact_i
    )
    return mtype, medias, ag
    
    
#===============================================================================
# MAIN
#===============================================================================

if __name__ == "__main__":
    print(__doc__)

