#!/usr/bin/env python
# -*- coding: utf-8 -*-

# "THE WISKEY-WARE LICENSE":
# <jbc.develop@gmail.com> wrote this file. As long as you retain this notice you
# can do whatever you want with this stuff. If we meet some day, and you think
# this stuff is worth it, you can buy me a WISKEY in return Juan BC

#===============================================================================
# DOC
#===============================================================================

"""Test for infopython.server

"""

#===============================================================================
# IMPORTS
#===============================================================================

import unittest
import random
import hashlib
import multiprocessing
import urllib2
import socket
import json

from infopython.util import xmlparser
from infopython import media, theory, server


#===============================================================================
# CONSTANTS
#===============================================================================

FORMATS = {
    "xml": (xmlparser.load, "text/xml"),
    "json": (json.load, "application/json"),
    "html": (lambda x: x.read(), "text/html")
}



#===============================================================================
# TEST
#===============================================================================

class ServerTests(unittest.TestCase):
    
    def setUp(self):
        
        def get_random_media_type():
            name = hashlib.sha1(
                str(random.random())
            ).hexdigest()
            return type(name, (media.Media,), {})
           
        def get_random_medias(mtype):
            buff = set()
            for _ in range(random.randint(2, 50)):
                name = hashlib.sha1(
                    str(random.random())
                ).hexdigest()
                buff.add(mtype(name))
            return list(buff)

        def unused_port():
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.bind(('localhost', 0))
            port = s.getsockname()[1]
            s.close()
            return port
        
        self.datas = {}
        for idx in range(random.randint(1, 100)):
            mtype = get_random_media_type()
            medias = get_random_medias(mtype)
            t = theory.Theory(str(idx), mtype, medias)
            self.datas[t.name] = t

        self.server = server.Server(theories=self.datas.values())
        
        self.port = unused_port()
        self.process = multiprocessing.Process(
            target=self.server.run,
            kwargs={"port": self.port}
        )
        self.process.start()

        # wait until process already started1
        started = False
        while not started:
            try:
                url = "http://127.0.0.1:{port}/{name}"
                urllib2.urlopen(
                    url.format(
                        name=self.server.name,
                        port=self.port
                    )
                ).read()
            except urllib2.HTTPError:
                started = True
            except urllib2.URLError as err:
                if err.reason.args[0] == 111:
                    pass
                else:
                    raise err
            else:
                started = True
        
    def tearDown(self):
        self.process.terminate()
    
    def iter_call(self, action, formats, *params):
        try:
            url = "http://127.0.0.1:{port}/{name}{action_and_params}{format}"
            action = "/" + action if action else ""
            for fmt in formats:
                parser, content_type = FORMATS[fmt]
                full_url = url.format(
                    port=self.port,
                    name=self.server.name,
                    action_and_params="/".join([action] + list(params)),
                    format="." + fmt if fmt != "html" else ""
                )
                
                response = urllib2.urlopen(full_url)
                
                # test response
                self.assertTrue(200 <= response.code and response.code <= 299)
                self.assertEqual(content_type, response.headers.type)
            
                # continue with normal test    
                yield fmt, parser(response)
        except:
            self.tearDown()
            raise 
        
    def test_index(self):
        for data in self.iter_call("", formats=FORMATS.keys()):
            if data[0] == "html":
                self.assertEqual(data[1], self.server.index_html())
            else:
                self.assertEqual(data[1], self.server.index())
                
    def test_list_theories(self):
        for data in self.iter_call("list_theories", ["xml", "json"]):
            for dt in data[1]:
                t = self.datas.get(dt["name"])
                self.assertIsInstance(t, theory.Theory)
                self.assertEqual(dt["type"], type(t).__name__)
                self.assertEqual(dt["media_type"], t.media_type.__name__)
    
    def test_show_theory(self):
        for tname, t in self.datas.items():
            for data in self.iter_call("show_theory", ["xml", "json"], tname):
                dt = data[1]
                t = self.datas.get(dt["name"])
                self.assertIsInstance(t, theory.Theory)
                self.assertEqual(dt["sorted_at"], None)
                self.assertEqual(dt["type"], type(t).__name__)
                self.assertEqual(dt["media_type"], t.media_type.__name__)
                for m in dt["medias"]:
                    self.assertIn(t.media_type(m), t)
                

#===============================================================================
# MAIN
#===============================================================================

if __name__ == "__main__":
    print(__doc__)


