#!/usr/bin/env python
# -*- coding: utf-8 -*-

# "THE WISKEY-WARE LICENSE":
# <jbc.develop@gmail.com> wrote this file. As long as you retain this notice you
# can do whatever you want with this stuff. If we meet some day, and you think
# this stuff is worth it, you can buy me a WISKEY in return Juan BC

#===============================================================================
# DOC
#===============================================================================

"""Test for infopython.theory

"""

#===============================================================================
# IMPORTS
#===============================================================================

import unittest
import random
import hashlib

from infopython import media, theory


#===============================================================================
# TEST
#===============================================================================

class TheoryTests(unittest.TestCase):
    
    def setUp(self):
        
        def get_random_media_type():
            name = hashlib.sha1(
                str(random.random())
            ).hexdigest()
            return type(name, (media.Media,), {})
           
        def get_random_medias(mtype):
            buff = set()
            for _ in range(random.randint(2, 100)):
                name = hashlib.sha1(
                    str(random.random())
                ).hexdigest()
                buff.add(mtype(name))
            return list(buff)
        
        self.mtype0 = get_random_media_type()
        self.medias0 = get_random_medias(self.mtype0)
        
        self.mtype1 = get_random_media_type()
        self.medias1 = get_random_medias(self.mtype1)
    
    def test_getitem(self):
        mt = self.mtype0
        medias = self.medias0
        t = theory.Theory("t", mt, medias)
        for idx in range(len(t)):
            self.assertIn(t[idx], medias)
    
    def test_str(self):
        mt = self.mtype0
        medias = self.medias0
        t = theory.Theory("t", mt, medias)
        self.assertEqual(repr(t), str(t))
    
    def test_repr(self):
        mt = self.mtype0
        medias = self.medias0
        t = theory.Theory("t", mt, medias)
        mrepr = "<{} '{}' (MType: {}) at {}>".format(
            t.__class__.__name__,
            "t",
            mt.__name__,
            hex(id(t)),
        )
        self.assertEqual(repr(t), mrepr)
    
    def test_add_contains_remove_iter_len(self):
        mt0 = self.mtype0
        mt1 = self.mtype1
        t0 = theory.Theory("t0", mt0, ())
        lent0 = 0
        for m in self.medias0:
            self.assertNotIn(m, t0)
            self.assertIsInstance(m, mt0)
            t0.add(m)
            self.assertIn(m, t0) # not works if contains fail
            lent0 += 1
        for m in self.medias1:
            self.assertNotIn(m, t0)
            self.assertIsInstance(m, mt1)
            self.assertRaises(AssertionError, t0.add, m)
            self.assertNotIn(m, t0)
        self.assertEquals(lent0, len(t0))
        
        t1 = theory.Theory("t1", mt1, ())
        lent1 = 0
        for m in self.medias1:
            self.assertNotIn(m, t1)
            self.assertIsInstance(m, mt1)
            t1.add(m)
            self.assertIn(m, t1)
            lent1 += 1
        for m in self.medias0:
            self.assertNotIn(m, t1)
            self.assertIsInstance(m, mt0)
            self.assertRaises(AssertionError, t1.add, m)
            self.assertNotIn(m, t1)
        self.assertEqual(lent1, len(t1))
            
        # removes
        for m in list(t0): # not works if iter fails
            self.assertIn(m, t0)
            t0.remove(m)
            self.assertNotIn(m, t0)
        self.assertEqual(0, len(t0))
        for m in list(t1): # not works if iter fails
            self.assertIn(m, t1)
            t1.remove(m)
            self.assertNotIn(m, t1)
        self.assertEquals(0, len(t1))

    def test_sort(self):
        mt = self.mtype0
        medias = self.medias0
        key = lambda m: m.name
        t = theory.Theory("t", mt, medias)
        self.assertEqual(medias, list(t))
        medias.sort(key=key)
        t.sort(key=key)
        self.assertEqual(medias, list(t))
        medias.sort(key=key, reverse=True)
        t.sort(key=key, reverse=True)
        self.assertEqual(medias, list(t))

    def test_clear(self):
        mt = self.mtype0
        medias = self.medias0
        t = theory.Theory("t", mt, medias)
        self.assertEqual(len(medias), len(t))
        t.clear()
        self.assertEqual(0, len(t))
        
    def test_index(self):
        mt = self.mtype0
        medias = self.medias0
        t = theory.Theory("t", mt, medias)
        for idx, m in enumerate(medias):
            self.assertEqual(idx, t.index(m))
        
        # reorder    
        key = lambda m: m.name
        medias.sort(key=key)
        t.sort(key=key)
        for idx, m in enumerate(medias):
            self.assertEqual(idx, t.index(m))
        medias.sort(key=key, reverse=True)
        t.sort(key=key, reverse=True)
        for idx, m in enumerate(medias):
            self.assertEqual(idx, t.index(m))
        
    def test_create_remove_media(self):
        mt = self.mtype0
        medias = self.medias0
        t = theory.Theory("t", mt, ())
        for m in medias:
            self.assertNotIn(m, t)
            t.create_media(name=m.name)
            self.assertIn(m, t)
        self.assertEqual(len(medias), len(t))
        for m in medias:
            self.assertIn(m, t)
            t.remove_media(name=m.name)
            self.assertNotIn(m, t)
        self.assertEqual(0, len(t))
            
    def test_dict_resume(self):
        mt = self.mtype0
        medias = self.medias0
        t = theory.Theory("t", mt, medias)
        d_resume = t.dict_resume()
        mresume = {
            "name": "t",
            "type": type(t).__name__,
            "media_type": mt.__name__,
            "datetime": d_resume["datetime"],
            "medias": tuple(m.name for m in medias),
            "sorted_at": d_resume["sorted_at"]
        }
        self.assertEqual(mresume, d_resume)

               
    
#===============================================================================
# MAIN
#===============================================================================

if __name__ == "__main__":
    print(__doc__)

